@@ -40,7 +40,6 @@
 #include "opentelemetry/sdk/metrics/meter_provider.h"
 #include "opentelemetry/sdk/trace/simple_processor_factory.h"
 #include "opentelemetry/sdk/trace/tracer_provider.h"
-#include "opentelemetry/trace/propagation/http_trace_context.h"
 
 #ifdef BAZEL_BUILD
 #include "examples/cpp/otel/util.h"
@@ -80,9 +79,8 @@ int main(int argc, char** argv) {
       grpc::OpenTelemetryPluginBuilder()
           .SetMeterProvider(std::move(meter_provider))
           .SetTracerProvider(std::move(tracer_provider))
-          .SetTextMapPropagator(
-              std::make_unique<
-                  opentelemetry::trace::propagation::HttpTraceContext>())
+          .SetTextMapPropagator(grpc::OpenTelemetryPluginBuilder::
+                                    MakeGrpcTraceBinTextMapPropagator())
           .BuildAndRegisterGlobal();
   if (!status.ok()) {
     std::cerr << "Failed to register gRPC OpenTelemetry Plugin: "
