@@ -21,7 +21,6 @@
 #include <grpc/grpc.h>
 #include <grpc/support/alloc.h>
 #include <grpc/support/string_util.h>
-#include <gtest/gtest.h>
 #include <openssl/crypto.h>
 #include <openssl/err.h>
 #include <openssl/pem.h>
@@ -29,8 +28,12 @@
 #include <stdio.h>
 #include <string.h>
 
+#include <string>
+#include <thread>
+
 #include "absl/log/log.h"
 #include "absl/strings/str_cat.h"
+#include "gtest/gtest.h"
 #include "src/core/tsi/transport_security.h"
 #include "src/core/tsi/transport_security_interface.h"
 #include "src/core/util/memory.h"
@@ -42,6 +45,7 @@
 #define SSL_TSI_TEST_ALPN1 "foo"
 #define SSL_TSI_TEST_ALPN2 "toto"
 #define SSL_TSI_TEST_ALPN3 "baz"
+#define SSL_TSI_TEST_ALPN4 "bar"
 #define SSL_TSI_TEST_ALPN_NUM 2
 #define SSL_TSI_TEST_SERVER_KEY_CERT_PAIRS_NUM 2
 #define SSL_TSI_TEST_BAD_SERVER_KEY_CERT_PAIRS_NUM 1
@@ -58,6 +62,8 @@ const size_t kSessionTicketEncryptionKeySize = 80;
 const size_t kSessionTicketEncryptionKeySize = 48;
 #endif
 
+constexpr static size_t kTls13FrameOverhead = 22;
+
 typedef enum AlpnMode {
   NO_ALPN,
   ALPN_CLIENT_NO_SERVER,
@@ -316,6 +322,21 @@ class SslTransportSecurityTest
       ssl_bio_buf_size_ = ssl_bio_buf_size;
     }
 
+    void OverrideHanshakerAlpnClientProtocols(
+        std::optional<std::string> alpn_client_protocols) {
+      alpn_client_overriden_protocols_ = alpn_client_protocols;
+    }
+
+    void OverrideHanshakerAlpnServerProtocols(
+        std::optional<std::string> alpn_server_protocols) {
+      alpn_server_overriden_protocols_ = alpn_server_protocols;
+    }
+
+    void SetExpectedAlpnNegotiatedProtocol(
+        std::optional<std::string> expected_alpn_negotiated_protocol) {
+      expected_alpn_negotiated_protocol_ = expected_alpn_negotiated_protocol;
+    }
+
    private:
     static void SetupHandshakers(tsi_test_fixture* fixture) {
       SslTsiTestFixture* ssl_fixture =
@@ -359,8 +380,17 @@ class SslTransportSecurityTest
       if (alpn_lib->alpn_mode == ALPN_SERVER_NO_CLIENT ||
           alpn_lib->alpn_mode == ALPN_CLIENT_SERVER_OK ||
           alpn_lib->alpn_mode == ALPN_CLIENT_SERVER_MISMATCH) {
-        server_options.alpn_protocols = alpn_lib->server_alpn_protocols;
-        server_options.num_alpn_protocols = alpn_lib->num_server_alpn_protocols;
+        if (ssl_fixture->alpn_server_overriden_protocols_.has_value()) {
+          size_t num_parsed_protocols = 0;
+          server_options.alpn_protocols = ParseAlpnStringIntoArray(
+              ssl_fixture->alpn_server_overriden_protocols_.value(),
+              static_cast<size_t*>(&num_parsed_protocols));
+          server_options.num_alpn_protocols = num_parsed_protocols;
+        } else {
+          server_options.alpn_protocols = alpn_lib->server_alpn_protocols;
+          server_options.num_alpn_protocols =
+              alpn_lib->num_server_alpn_protocols;
+        }
         if (alpn_lib->alpn_mode == ALPN_CLIENT_SERVER_MISMATCH) {
           server_options.num_alpn_protocols--;
         }
@@ -397,12 +427,21 @@ class SslTransportSecurityTest
       ASSERT_EQ(tsi_create_ssl_server_handshaker_factory_with_options(
                     &server_options, &ssl_fixture->server_handshaker_factory_),
                 TSI_OK);
+      if (ssl_fixture->alpn_server_overriden_protocols_.has_value()) {
+        // Free memory from creating the server protocol list if
+        // ParseAlpnStringIntoArray was used.
+        for (size_t i = 0; i < server_options.num_alpn_protocols; i++) {
+          gpr_free(const_cast<char*>(server_options.alpn_protocols[i]));
+        }
+        gpr_free(server_options.alpn_protocols);
+      }
       // Create server and client handshakers.
       ASSERT_EQ(tsi_ssl_client_handshaker_factory_create_handshaker(
                     ssl_fixture->client_handshaker_factory_,
                     ssl_fixture->server_name_indication_,
                     ssl_fixture->network_bio_buf_size_,
                     ssl_fixture->ssl_bio_buf_size_,
+                    ssl_fixture->alpn_client_overriden_protocols_,
                     &ssl_fixture->base_.client_handshaker),
                 TSI_OK);
       ASSERT_EQ(tsi_ssl_server_handshaker_factory_create_handshaker(
@@ -424,10 +463,16 @@ class SslTransportSecurityTest
         ASSERT_EQ(alpn_property, nullptr);
       } else {
         ASSERT_NE(alpn_property, nullptr);
-        const char* expected_match = "baz";
-        ASSERT_EQ(memcmp(alpn_property->value.data, expected_match,
-                         alpn_property->value.length),
-                  0);
+        std::string expected_match;
+        if (ssl_fixture->expected_alpn_negotiated_protocol_.has_value()) {
+          expected_match =
+              ssl_fixture->expected_alpn_negotiated_protocol_.value();
+        } else {
+          expected_match = "baz";
+        }
+        ASSERT_EQ(
+            std::string(alpn_property->value.data, alpn_property->value.length),
+            expected_match);
       }
     }
 
@@ -655,6 +700,10 @@ class SslTransportSecurityTest
     bool verify_root_cert_subject_;
     tsi_tls_version tls_version_;
     bool send_client_ca_list_;
+    std::optional<std::string> alpn_client_overriden_protocols_ = std::nullopt;
+    std::optional<std::string> alpn_server_overriden_protocols_ = std::nullopt;
+    std::optional<std::string> expected_alpn_negotiated_protocol_ =
+        std::nullopt;
     tsi_ssl_server_handshaker_factory* server_handshaker_factory_ = nullptr;
     tsi_ssl_client_handshaker_factory* client_handshaker_factory_ = nullptr;
   };
@@ -1015,6 +1064,124 @@ TEST_P(SslTransportSecurityTest, DoHandshakeWithCustomBioPair) {
   DoHandshake();
 }
 
+// TODO(matthewstevenson88): Make tests below compatible with OpenSSL.
+#if defined(OPENSSL_IS_BORINGSSL)
+TEST_P(SslTransportSecurityTest, Protect) {
+  LOG(INFO) << "ssl_tsi_test_protect";
+  SetUpSslFixture(tsi_tls_version::TSI_TLS1_3, /*send_client_ca_list=*/false);
+  DoHandshake();
+  tsi_frame_protector* protector;
+  EXPECT_EQ(tsi_handshaker_result_create_frame_protector(
+                ssl_tsi_test_fixture_->client_result,
+                /*max_output_protected_frame_size=*/nullptr, &protector),
+            TSI_OK);
+  ASSERT_NE(protector, nullptr);
+  std::string buffer(1024, 'a');
+  std::string protected_bytes = Protect(protector, buffer);
+  EXPECT_EQ(protected_bytes.size(), buffer.size() + kTls13FrameOverhead);
+  tsi_frame_protector_destroy(protector);
+}
+
+TEST_P(SslTransportSecurityTest, ProtectAndUnprotect) {
+  LOG(INFO) << "ssl_tsi_test_protect_and_unprotect";
+  SetUpSslFixture(tsi_tls_version::TSI_TLS1_3, /*send_client_ca_list=*/false);
+  DoHandshake();
+  tsi_frame_protector* client_protector;
+  EXPECT_EQ(tsi_handshaker_result_create_frame_protector(
+                ssl_tsi_test_fixture_->client_result,
+                /*max_output_protected_frame_size=*/nullptr, &client_protector),
+            TSI_OK);
+  ASSERT_NE(client_protector, nullptr);
+  tsi_frame_protector* server_protector;
+  EXPECT_EQ(tsi_handshaker_result_create_frame_protector(
+                ssl_tsi_test_fixture_->server_result,
+                /*max_output_protected_frame_size=*/nullptr, &server_protector),
+            TSI_OK);
+  ASSERT_NE(server_protector, nullptr);
+  std::string buffer(1024, 'a');
+  std::string protected_bytes = Protect(client_protector, buffer);
+  std::string unprotected_bytes = Unprotect(server_protector, protected_bytes);
+  EXPECT_EQ(unprotected_bytes, buffer);
+  tsi_frame_protector_destroy(client_protector);
+  tsi_frame_protector_destroy(server_protector);
+}
+
+TEST_P(SslTransportSecurityTest, ConcurrentlyProtectAndUnprotectOnClient) {
+  LOG(INFO) << "ssl_tsi_test_protect_and_unprotect";
+  SetUpSslFixture(tsi_tls_version::TSI_TLS1_3, /*send_client_ca_list=*/false);
+  DoHandshake();
+  tsi_frame_protector* client_protector;
+  EXPECT_EQ(tsi_handshaker_result_create_frame_protector(
+                ssl_tsi_test_fixture_->client_result,
+                /*max_output_protected_frame_size=*/nullptr, &client_protector),
+            TSI_OK);
+  ASSERT_NE(client_protector, nullptr);
+  tsi_frame_protector* server_protector;
+  EXPECT_EQ(tsi_handshaker_result_create_frame_protector(
+                ssl_tsi_test_fixture_->server_result,
+                /*max_output_protected_frame_size=*/nullptr, &server_protector),
+            TSI_OK);
+  ASSERT_NE(server_protector, nullptr);
+  std::string buffer(1024, 'a');
+  std::string protected_bytes = Protect(server_protector, buffer);
+  std::thread protect_thread([&client_protector]() {
+    std::string second_buffer(2048, 'b');
+    std::string second_protected_buffer =
+        Protect(client_protector, second_buffer);
+    EXPECT_EQ(second_protected_buffer.size(),
+              second_buffer.size() + kTls13FrameOverhead);
+  });
+  std::thread unprotect_thread([&client_protector, protected_bytes, buffer]() {
+    std::string unprotected_bytes =
+        Unprotect(client_protector, protected_bytes);
+    EXPECT_EQ(unprotected_bytes, buffer);
+  });
+  protect_thread.join();
+  unprotect_thread.join();
+  tsi_frame_protector_destroy(client_protector);
+  tsi_frame_protector_destroy(server_protector);
+}
+
+TEST_P(SslTransportSecurityTest, ConcurrentlyProtectAndUnprotectOnServer) {
+  LOG(INFO) << "ssl_tsi_test_protect_and_unprotect";
+  SetUpSslFixture(tsi_tls_version::TSI_TLS1_3, /*send_client_ca_list=*/false);
+  DoHandshake();
+  tsi_frame_protector* client_protector;
+  EXPECT_EQ(tsi_handshaker_result_create_frame_protector(
+                ssl_tsi_test_fixture_->client_result,
+                /*max_output_protected_frame_size=*/nullptr, &client_protector),
+            TSI_OK);
+  ASSERT_NE(client_protector, nullptr);
+  tsi_frame_protector* server_protector;
+  EXPECT_EQ(tsi_handshaker_result_create_frame_protector(
+                ssl_tsi_test_fixture_->server_result,
+                /*max_output_protected_frame_size=*/nullptr, &server_protector),
+            TSI_OK);
+  ASSERT_NE(server_protector, nullptr);
+  std::string buffer(1024, 'a');
+  std::string protected_bytes = Protect(client_protector, buffer);
+  std::thread protect_thread([&server_protector]() {
+    std::string second_buffer(20, 'b');
+    std::string second_protected_buffer =
+        Protect(server_protector, second_buffer);
+    // The protected bytes may be prefixed by a TLS record containing session
+    // tickets, so the total number of protected bytes is greater or equal to
+    // the expected bytes, based on the length of the plaintext.
+    EXPECT_GE(second_protected_buffer.size(),
+              second_buffer.size() + kTls13FrameOverhead);
+  });
+  std::thread unprotect_thread([&server_protector, protected_bytes, buffer]() {
+    std::string unprotected_bytes =
+        Unprotect(server_protector, protected_bytes);
+    EXPECT_GE(unprotected_bytes, buffer);
+  });
+  protect_thread.join();
+  unprotect_thread.join();
+  tsi_frame_protector_destroy(client_protector);
+  tsi_frame_protector_destroy(server_protector);
+}
+#endif  // defined(OPENSSL_IS_BORINGSSL)
+
 static const tsi_ssl_handshaker_factory_vtable* original_vtable;
 static bool handshaker_factory_destructor_called;
 
@@ -1051,7 +1218,8 @@ TEST(SslTransportSecurityTest, TestClientHandshakerFactoryRefcounting) {
   for (i = 0; i < 3; ++i) {
     ASSERT_EQ(
         tsi_ssl_client_handshaker_factory_create_handshaker(
-            client_handshaker_factory, "google.com", 0, 0, &handshaker[i]),
+            client_handshaker_factory, "google.com", 0, 0,
+            /*alpn_preferred_protocol_list=*/std::nullopt, &handshaker[i]),
         TSI_OK);
   }
 
@@ -1294,6 +1462,42 @@ TEST(SslTransportSecurityTest, ExtractCertChain) {
   sk_X509_pop_free(cert_chain, X509_free);
 }
 
+// Attempt to perform a handshake between a client and server with ALPN enabled
+// and overriding the preferred protocols on the handshaker factory.
+TEST_P(SslTransportSecurityTest, TestClientAndServerHandshakerOverrideALPN) {
+  SetUpSslFixture(/*tls_version=*/std::get<0>(GetParam()),
+                  /*send_client_ca_list=*/std::get<1>(GetParam()));
+  ssl_fixture_->OverrideHanshakerAlpnClientProtocols("toto,bar");
+  ssl_fixture_->OverrideHanshakerAlpnServerProtocols("bar,foo,toto");
+  ssl_fixture_->SetExpectedAlpnNegotiatedProtocol("bar");
+  ssl_fixture_->SetAlpnMode(ALPN_CLIENT_SERVER_OK);
+  DoHandshake();
+}
+
+// Attempt to perform a handshake between a client and server with ALPN enabled
+// and overriding the preferred protocols on the handshaker factory.
+TEST_P(SslTransportSecurityTest, TestClientHandshakerOverrideALPN) {
+  SetUpSslFixture(/*tls_version=*/std::get<0>(GetParam()),
+                  /*send_client_ca_list=*/std::get<1>(GetParam()));
+  ssl_fixture_->OverrideHanshakerAlpnClientProtocols("toto,baz");
+  // The server default protocols set by the handshaker factory are [foo, baz].
+  ssl_fixture_->SetExpectedAlpnNegotiatedProtocol("baz");
+  ssl_fixture_->SetAlpnMode(ALPN_CLIENT_SERVER_OK);
+  DoHandshake();
+}
+
+// Attempt to perform a handshake between a client and server with ALPN enabled
+// and overriding the preferred protocols on the handshaker factory.
+TEST_P(SslTransportSecurityTest, TestServerHandshakerOverrideALPN) {
+  SetUpSslFixture(/*tls_version=*/std::get<0>(GetParam()),
+                  /*send_client_ca_list=*/std::get<1>(GetParam()));
+  // The client default protocols set by the handshaker factory are [toto, baz].
+  ssl_fixture_->OverrideHanshakerAlpnServerProtocols("bar,foo,toto");
+  ssl_fixture_->SetExpectedAlpnNegotiatedProtocol("toto");
+  ssl_fixture_->SetAlpnMode(ALPN_CLIENT_SERVER_OK);
+  DoHandshake();
+}
+
 int main(int argc, char** argv) {
   grpc::testing::TestEnvironment env(&argc, argv);
   ::testing::InitGoogleTest(&argc, argv);
