@@ -16,8 +16,7 @@
 //
 //
 
-#include <gtest/gtest.h>
-
+#include "gtest/gtest.h"
 #include "test/core/test_util/test_config.h"
 
 namespace grpc {
