@@ -21,7 +21,12 @@ import ctypes
 import math
 import sys
 
-_REQUIRED_FIELDS = ["name", "doc", "scope"]
+_REQUIRED_FIELDS = [
+    "name",
+    "doc",
+    "scope",
+    "linked_global_scope",
+]
 
 
 def make_type(name, fields, defaults):
@@ -141,9 +146,11 @@ def histogram_shape(histogram, global_scope):
     else:
         return Shape(
             histogram.max,
-            histogram.scope_buckets
-            if histogram.scope_buckets
-            else histogram.buckets,
+            (
+                histogram.scope_buckets
+                if histogram.scope_buckets
+                else histogram.buckets
+            ),
             histogram.scope_counter_bits,
         )
 
@@ -152,6 +159,80 @@ def histogram_shape_signature(histogram, global_scope):
     return shape_signature(histogram_shape(histogram, global_scope))
 
 
+class DefaultLocalScopedStatsCollectorGenerator:
+    """Generate StatsCollector classes for a given local scope."""
+
+    def __init__(self, scope, linked_global_scope="global"):
+        self._scope = scope
+        self._linked_global_scope = linked_global_scope
+
+    def generate_stats_collector(self, class_name, inst_map, H):
+        print(" public:", file=H)
+        print(
+            "  const %s& View() const { return data_; };" % class_name,
+            file=H,
+        )
+        for ctr in inst_map["Counter"]:
+            if (
+                ctr.scope != self._scope
+                and ctr.scope != self._linked_global_scope
+            ):
+                continue
+            if ctr.scope == self._linked_global_scope:
+                print(
+                    "  void Increment%s() { %s_stats().Increment%s(); }"
+                    % (
+                        snake_to_pascal(ctr.name),
+                        self._linked_global_scope,
+                        snake_to_pascal(ctr.name),
+                    ),
+                    file=H,
+                )
+            else:
+                print(
+                    "  void Increment%s() { ++data_.%s; %s_stats().Increment%s(); }"
+                    % (
+                        snake_to_pascal(ctr.name),
+                        ctr.name,
+                        self._linked_global_scope,
+                        snake_to_pascal(ctr.name),
+                    ),
+                    file=H,
+                )
+        for ctr in inst_map["Histogram"]:
+            if (
+                ctr.scope != self._scope
+                and ctr.scope != self._linked_global_scope
+            ):
+                continue
+            if ctr.scope == self._linked_global_scope:
+                print(
+                    "  void Increment%s(int value) { "
+                    " %s_stats().Increment%s(value); }"
+                    % (
+                        snake_to_pascal(ctr.name),
+                        self._linked_global_scope,
+                        snake_to_pascal(ctr.name),
+                    ),
+                    file=H,
+                )
+            else:
+                print(
+                    "  void Increment%s(int value) { data_.%s.Increment(value);"
+                    " %s_stats().Increment%s(value); }"
+                    % (
+                        snake_to_pascal(ctr.name),
+                        ctr.name,
+                        self._linked_global_scope,
+                        snake_to_pascal(ctr.name),
+                    ),
+                    file=H,
+                )
+
+        print(" private:", file=H)
+        print("  %s data_;" % class_name, file=H)
+
+
 class StatsDataGenerator:
     """Generates stats_data.h and stats_data.cc."""
 
@@ -169,6 +250,7 @@ class StatsDataGenerator:
         self._static_tables = []
         self._shapes = set()
         self._scopes = set()
+        self._linked_global_scopes = {}
         for attr in self._attrs:
             found = False
             for t, lst in self._types:
@@ -178,17 +260,35 @@ class StatsDataGenerator:
                     del attr[t_name]
                     lst.append(t(name=name, **attr))
                     self._scopes.add(attr["scope"])
+                    self._linked_global_scopes[attr["scope"]] = attr[
+                        "linked_global_scope"
+                    ]
                     found = True
                     break
             assert found, "Bad decl: %s" % attr
         for histogram in self._inst_map["Histogram"]:
             self._shapes.add(histogram_shape(histogram, True))
-            if histogram.scope != "global":
+            if histogram.scope != histogram.linked_global_scope:
                 self._shapes.add(histogram_shape(histogram, False))
         # make self._scopes a sorted list, but with global at the start
         assert "global" in self._scopes
-        self._scopes.remove("global")
-        self._scopes = ["global"] + sorted(self._scopes)
+        global_scopes = []
+        for scope in self._scopes:
+            if scope == self._linked_global_scopes[scope]:
+                global_scopes.append(scope)
+        for scope in global_scopes:
+            self._scopes.remove(scope)
+        self._scopes = sorted(global_scopes) + sorted(self._scopes)
+        self._scoped_stats_collector_generators = {}
+
+    def register_scoped_stats_collector_generator(
+        self, scope, linked_global_scope, generator
+    ):
+        if scope not in self._scoped_stats_collector_generators:
+            self._scoped_stats_collector_generators[scope] = {}
+        self._scoped_stats_collector_generators[scope][
+            linked_global_scope
+        ] = generator
 
     def _decl_static_table(self, values, type):
         v = (type, values)
@@ -433,8 +533,10 @@ class StatsDataGenerator:
                     print("};", file=H)
 
             for scope in self._scopes:
+                linked_global_scope = self._linked_global_scopes[scope]
                 include_ctr = (
-                    lambda ctr: scope == "global" or ctr.scope == scope
+                    lambda ctr: ctr.scope == scope
+                    or ctr.linked_global_scope == scope
                 )
 
                 class_name = snake_to_pascal(scope) + "Stats"
@@ -500,12 +602,15 @@ class StatsDataGenerator:
                     print(
                         "  Histogram_%s %s;"
                         % (
-                            histogram_shape_signature(ctr, scope == "global"),
+                            histogram_shape_signature(
+                                ctr, scope == linked_global_scope
+                            ),
                             ctr.name,
                         ),
                         file=H,
                     )
-                if scope == "global":
+                # Check if the scope is of type 'global'
+                if scope == linked_global_scope:
                     print(
                         "  HistogramView histogram(Histogram which) const;",
                         file=H,
@@ -518,7 +623,7 @@ class StatsDataGenerator:
                 print("};", file=H)
 
                 print("class %sCollector {" % class_name, file=H)
-                if scope == "global":
+                if scope == linked_global_scope:
                     print(" public:", file=H)
                     print(
                         "  std::unique_ptr<%s> Collect() const;" % class_name,
@@ -539,7 +644,7 @@ class StatsDataGenerator:
                     for ctr in self._inst_map["Counter"]:
                         if not include_ctr(ctr):
                             continue
-                        set_private(ctr.scope != "global")
+                        set_private(ctr.scope != scope)
                         print(
                             "  void Increment%s() { data_.this_cpu().%s.fetch_add(1,"
                             " std::memory_order_relaxed); }"
@@ -549,7 +654,7 @@ class StatsDataGenerator:
                     for ctr in self._inst_map["Histogram"]:
                         if not include_ctr(ctr):
                             continue
-                        set_private(ctr.scope != "global")
+                        set_private(ctr.scope != scope)
                         print(
                             "  void Increment%s(int value) {"
                             " data_.this_cpu().%s.Increment(value); }"
@@ -558,7 +663,7 @@ class StatsDataGenerator:
                         )
                     set_private(True)
                     for other_scope in self._scopes:
-                        if other_scope == "global":
+                        if other_scope == scope:
                             continue
                         print(
                             "  friend class %sStatsCollector;"
@@ -580,7 +685,7 @@ class StatsDataGenerator:
                             "    HistogramCollector_%s %s;"
                             % (
                                 histogram_shape_signature(
-                                    ctr, scope == "global"
+                                    ctr, scope == linked_global_scope
                                 ),
                                 ctr.name,
                             ),
@@ -595,45 +700,32 @@ class StatsDataGenerator:
                         file=H,
                     )
                 else:  # not global
-                    print(" public:", file=H)
+                    if scope not in self._scoped_stats_collector_generators:
+                        generator = DefaultLocalScopedStatsCollectorGenerator(
+                            scope, linked_global_scope
+                        )
+                    else:
+                        generator = self._scoped_stats_collector_generators[
+                            scope
+                        ][linked_global_scope]
+                    generator.generate_stats_collector(
+                        class_name, self._inst_map, H
+                    )
+
+                print("};", file=H)
+                if scope == linked_global_scope:
                     print(
-                        "  const %s& View() const { return data_; };"
-                        % class_name,
+                        "inline %sStatsCollector& %s_stats() {"
+                        % (
+                            snake_to_pascal(scope),
+                            scope,
+                        ),
                         file=H,
                     )
-                    for ctr in self._inst_map["Counter"]:
-                        if not include_ctr(ctr):
-                            continue
-                        print(
-                            "  void Increment%s() { ++data_.%s; global_stats().Increment%s(); }"
-                            % (
-                                snake_to_pascal(ctr.name),
-                                ctr.name,
-                                snake_to_pascal(ctr.name),
-                            ),
-                            file=H,
-                        )
-                    for ctr in self._inst_map["Histogram"]:
-                        if not include_ctr(ctr):
-                            continue
-                        print(
-                            "  void Increment%s(int value) { data_.%s.Increment(value); global_stats().Increment%s(value); }"
-                            % (
-                                snake_to_pascal(ctr.name),
-                                ctr.name,
-                                snake_to_pascal(ctr.name),
-                            ),
-                            file=H,
-                        )
-                    print(" private:", file=H)
-                    print("  %s data_;" % class_name, file=H)
-                print("};", file=H)
-                if scope == "global":
                     print(
-                        "inline GlobalStatsCollector& global_stats() {", file=H
-                    )
-                    print(
-                        "  return *NoDestructSingleton<GlobalStatsCollector>::Get();",
+                        "  return"
+                        " *NoDestructSingleton<%sStatsCollector>::Get();"
+                        % snake_to_pascal(scope),
                         file=H,
                     )
                     print("}", file=H)
@@ -755,8 +847,10 @@ class StatsDataGenerator:
                 )
 
             for scope in self._scopes:
+                linked_global_scope = self._linked_global_scopes[scope]
                 include_ctr = (
-                    lambda ctr: scope == "global" or ctr.scope == scope
+                    lambda ctr: ctr.scope == scope
+                    or ctr.linked_global_scope == scope
                 )
                 class_name = snake_to_pascal(scope) + "Stats"
                 for typename, instances in sorted(self._inst_map.items()):
@@ -797,7 +891,7 @@ class StatsDataGenerator:
                     file=C,
                 )
 
-                if scope == "global":
+                if scope == linked_global_scope:
                     print(
                         "HistogramView %s::histogram(Histogram which) const {"
                         % class_name,
@@ -816,7 +910,9 @@ class StatsDataGenerator:
                             % snake_to_pascal(inst.name),
                             file=C,
                         )
-                        shape = histogram_shape(inst, scope == "global")
+                        shape = histogram_shape(
+                            inst, scope == linked_global_scope
+                        )
                         print(
                             "      return HistogramView{&Histogram_%s::BucketFor,"
                             " kStatsTable%d, %d, %s.buckets()};"
@@ -831,48 +927,85 @@ class StatsDataGenerator:
                     print("  }", file=C)
                     print("}", file=C)
 
-            print(
-                "std::unique_ptr<GlobalStats> GlobalStatsCollector::Collect()"
-                " const {",
-                file=C,
-            )
-            print("  auto result = std::make_unique<GlobalStats>();", file=C)
-            print("  for (const auto& data : data_) {", file=C)
-            for ctr in self._inst_map["Counter"]:
-                print(
-                    "    result->%s += data.%s.load(std::memory_order_relaxed);"
-                    % (ctr.name, ctr.name),
-                    file=C,
-                )
-            for h in self._inst_map["Histogram"]:
-                print(
-                    "    data.%s.Collect(&result->%s);" % (h.name, h.name),
-                    file=C,
-                )
-            print("  }", file=C)
-            print("  return result;", file=C)
-            print("}", file=C)
+                    print(
+                        "std::unique_ptr<%sStats> %sStatsCollector::Collect()"
+                        " const {"
+                        % (snake_to_pascal(scope), snake_to_pascal(scope)),
+                        file=C,
+                    )
+                    print(
+                        "  auto result = std::make_unique<%sStats>();"
+                        % snake_to_pascal(scope),
+                        file=C,
+                    )
+                    print("  for (const auto& data : data_) {", file=C)
+                    for ctr in self._inst_map["Counter"]:
+                        if (
+                            ctr.scope != scope
+                            and self._linked_global_scopes[ctr.scope] != scope
+                        ):
+                            continue
+                        print(
+                            "    result->%s +="
+                            " data.%s.load(std::memory_order_relaxed);"
+                            % (ctr.name, ctr.name),
+                            file=C,
+                        )
+                    for h in self._inst_map["Histogram"]:
+                        if (
+                            h.scope != scope
+                            and self._linked_global_scopes[h.scope] != scope
+                        ):
+                            continue
+                        print(
+                            "    data.%s.Collect(&result->%s);"
+                            % (h.name, h.name),
+                            file=C,
+                        )
+                    print("  }", file=C)
+                    print("  return result;", file=C)
+                    print("}", file=C)
 
-            print(
-                (
-                    "std::unique_ptr<GlobalStats> GlobalStats::Diff(const"
-                    " GlobalStats& other) const {"
-                ),
-                file=C,
-            )
-            print("  auto result = std::make_unique<GlobalStats>();", file=C)
-            for ctr in self._inst_map["Counter"]:
-                print(
-                    "  result->%s = %s - other.%s;"
-                    % (ctr.name, ctr.name, ctr.name),
-                    file=C,
-                )
-            for h in self._inst_map["Histogram"]:
-                print(
-                    "  result->%s = %s - other.%s;" % (h.name, h.name, h.name),
-                    file=C,
-                )
-            print("  return result;", file=C)
-            print("}", file=C)
+                    print(
+                        (
+                            "std::unique_ptr<%sStats> %sStats::Diff(const"
+                            " %sStats& other) const {"
+                        )
+                        % (
+                            snake_to_pascal(scope),
+                            snake_to_pascal(scope),
+                            snake_to_pascal(scope),
+                        ),
+                        file=C,
+                    )
+                    print(
+                        "  auto result = std::make_unique<%sStats>();"
+                        % snake_to_pascal(scope),
+                        file=C,
+                    )
+                    for ctr in self._inst_map["Counter"]:
+                        if (
+                            ctr.scope != scope
+                            and self._linked_global_scopes[ctr.scope] != scope
+                        ):
+                            continue
+                        print(
+                            "  result->%s = %s - other.%s;"
+                            % (ctr.name, ctr.name, ctr.name),
+                            file=C,
+                        )
+                    for h in self._inst_map["Histogram"]:
+                        if (
+                            h.scope != scope
+                            and self._linked_global_scopes[h.scope] != scope
+                        ):
+                            continue
+                        print(
+                            "  result->%s = %s - other.%s;"
+                            % (h.name, h.name, h.name),
+                            file=C,
+                        )
+                    print("  return result;", file=C)
+                    print("}", file=C)
 
             print("}", file=C)
